package sun.beanbox;

import java.io.*;
import java.util.*;
import java.beans.Introspector;

/**
 * Generate a AppletClass for an Applet.
 */

public class AppletClassGenerator {

    /**
     * A generator for AppletClasses.
     * Box is the beanbox
     * Name is the AppletClass name
     * Dir is the directory where to place the generated .java and .class files
     */
    public AppletClassGenerator(BeanBox box) {
	beanBox = box;
    }

    /**
     * Generate a AppletClass for the given BeanBox;
     * Includes compiling it
     *
     * TBD; classes should not be in the default package
     * we should pass around the package name.
     */
    public boolean generateClass(String appletName, File dir, String[] jars) {
	String fileName = generateJavaFile(appletName, dir);

	// create the desired classpath
	StringBuffer classpath = new StringBuffer(dir.getPath());
	for (int i=0; i<jars.length; i++) {
	    classpath.append(File.pathSeparator);
	    classpath.append(jars[i]);
	}
	classpath.append(File.pathSeparator);
	classpath.append(System.getProperty("java.class.path"));
	
	return ClassCompiler.compile(fileName, classpath.toString());
    }

    /**
     * Write the .java file
     */
    private String generateJavaFile(String appletName, File dir) {
	File f = new File(dir, appletName + ".java");
	try {
	    p = new IndentedStream(new PrintWriter(new FileOutputStream(f)));
	} catch (Exception ex) {
	    System.err.println("could not create output stream for codegen; "+ex);
	    System.exit(1);
	}

	p.pp("import java.io.Serializable;");
	p.pp("import java.io.InputStream;");
	p.pp("import java.io.OutputStream;");
	p.pp("import java.io.ObjectOutputStream;");
	p.pp("import java.io.ObjectInputStream;");
	p.pp("import java.awt.Component;");
	p.pp("import java.awt.Rectangle;");
	p.pp("import java.awt.Dimension;");
	p.pp("import java.applet.Applet;");
	p.pp("import sunw.beanbox.AppletSupport;");

	p.pp("");
	p.pp("public class "+appletName+" extends Applet implements Serializable {");
	p.i();
	int count = beanBox.getComponentCount();

	p.pp();
	p.pp("// Public 0-argument constructor");
	p.pp();

	p.pp("public "+appletName+"() {");
	p.ip("InputStream is = this.getClass().getResourceAsStream(\""+
	   appletName+"Data\");");
	p.pp("if (is == null) {");
	p.ip("System.err.println(\"Could not locate the Applet data at "+
	     appletName+"Data\");");
	p.pp("throw new Error(\"Could not locate the Applet data at "+
	     appletName+"Data\");");
	p.op("}");
	p.pp("this.setLayout(null);");
	p.pp("try {");
	p.ip("ObjectInputStream ois = new ObjectInputStream(is);");
	p.pp("Object[] data = (Object[]) ois.readObject();");
	p.pp("Rectangle[] boundsData = (Rectangle[]) ois.readObject();");

	p.pp();
	p.pp("String id = (String) data[0];");;
	p.pp("if (! id.equals(\""+appletName+"\")) {");
	p.ip("throw new Error(\"Wrong data!\");");
	p.op("}");

	p.pp("Rectangle b = boundsData[0];");
	p.pp("mySize = new Dimension(b.width, b.height);");
	p.pp("this.setSize(mySize);");
	p.pp();

	// generate based on this BeanBox...
	for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) beanBox.getComponent(i);
	    Object bean = w.getBean();
	    readIntoField(bean, i+1, true);
	} 

	p.op("} catch (Exception ex) {");
	p.ip("System.err.println(\"trouble reading from serialized data: \"+ex);");
	p.ip("throw new Error(\"trouble reading from serialized data: \"+ex);");
	p.op("}");
	p.op("}");

	p.pp();
	p.pp("public Dimension getPreferredSize() {");
	p.ip("return mySize;");
	p.op("}");

	p.pp();
	p.pp("// support code");
	generateSupportCode();

	p.pp();
	p.pp("// Serialization code");
        p.pp("private void readObject(java.io.ObjectInputStream ois)");
        p.ip("throws java.lang.ClassNotFoundException,");
        p.pp("java.io.IOException");
	p.op("{");
	p.i();

	p.pp("Object[] data = (Object[]) ois.readObject();");
	p.pp("mySize = (Dimension) ois.readObject();");
	p.pp();
	p.pp("String id = (String) data[0];");;
	p.pp("if (! id.equals(\""+appletName+"\")) {");
	p.ip("throw new Error(\"Wrong data!\");");
	p.op("}");

	// generate based on this BeanBox...
	for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) beanBox.getComponent(i);
	    Object bean = w.getBean();
	    readIntoField(bean, i+1, false);
	} 
	p.op("}");
	p.pp();

        p.pp("private void writeObject(java.io.ObjectOutputStream oos)");
       	p.ip("throws java.io.IOException");
        p.op("{");
	p.i();

	p.pp("Object data[] = new Object["+(count+1)+"];");
	p.pp("data[0] = \""+appletName+"\";");
	// generate based on this BeanBox...
	for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) beanBox.getComponent(i);
	    Object bean = w.getBean();
	    String name = uniqueName(bean);
            p.pp("data["+(i+1)+"] = "+name+";");
	} 
	p.pp("oos.writeObject(data);");
	p.pp("oos.writeObject(mySize);");
	p.op("}");
	p.pp();

	// based on this BeanBox
	p.pp();
	p.pp("// The fields used to hold the beans");
	for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) beanBox.getComponent(i);
	    Object bean = w.getBean();
	    p.pp("private "+bean.getClass().getName()+" "+uniqueName(bean)+";");
	}

    	p.pp();
	p.pp("// the loader so we can locate the resource file");
	p.pp("private ClassLoader myLoader = this.getClass().getClassLoader();");

	p.pp();
	p.pp("private Dimension mySize;");

	// All Done...
	p.op("}");
	p.close();
	
	return f.getPath();
    }

    private void readIntoField(Object bean, int ix, boolean setBounds) {
        Class cl = bean.getClass();
	String clname = cl.getName();
	String name = uniqueName(bean);
        p.pp(name+" = ("+clname+") data["+ix+"];");
	if (setBounds) {
	    p.pp("acquire("+name+", boundsData["+ix+"]);");
	} else {
	    p.pp("acquire("+name+", null);");
	}
    }

    private void generateSupportCode() {
	p.pp("// acquire a bean");
	p.pp("private void acquire(Object bean, Rectangle boundsData) {");
	p.ip("if (!(bean instanceof Component)) {");
	p.ip("return;");
	p.op("}");
	p.pp("if (bean instanceof Applet) {");
        p.ip("AppletSupport.assignStub((Applet) bean,");
	p.ip("myLoader,");
	p.op("bean.getClass());");
	p.op("}");
	p.pp();
	p.pp("add((Component) bean);");
	p.pp("if (boundsData != null) {");
	p.ip("((Component) bean).setBounds(boundsData);");
	p.op("}");
	p.pp("((Component)bean).invalidate();	// not needed?");
	p.pp();
	p.pp("if (bean instanceof Applet) {");
	p.ip("// Start the Applet");
	p.pp("((Applet)bean).start();");
	p.op("}");
	p.op("}");
    }

    /**
     *  manage unique names
     */

    private String uniqueName(Object bean) {
	String back = (String) names.get(bean);
	if (back != null) {
	    return back; // the name we last assigned it
	}
	String klassName = bean.getClass().getName();
	int ix = klassName.lastIndexOf('.');
	if (ix >= 0) {
	    klassName = klassName.substring(ix+1);
	}
	// decapitalize
	klassName = Introspector.decapitalize(klassName);
	// note that different classes may have the same short name
	Integer klassCount = (Integer) names.get(klassName);

	int count = 0; // last count for this class name
	if (klassCount != null) {
	    count = klassCount.intValue();
	}
	count += 1;
	back = klassName+count;
	names.put(klassName, new Integer(count));
	names.put(bean, back);
        return back;
    }

    // Private fields for Class generation

    private Hashtable names = new Hashtable(); // unique names
    private BeanBox beanBox;	// source for what we want to pack
    private String appletName;	// applet name
    private static File dir;	// where to generate before packing it up

    private IndentedStream p = null; // the stream for the code
}
